@@ -27,4 +27,5 @@
 /* Define if you have the strsignal function.  */
 #undef HAVE_STRSIGNAL
 
-
+/* Define if your hosts data base have a non-standard path.  */
+#undef HOSTS_PATH
