@@ -1,5 +1,5 @@
 /* Asp Address Search Protocol Client (file: asp.c)                        */
-/* Copyright (C) 1996-97 Stenio Brunetta                                   */
+/* Copyright (C) 1996-2001 Stenio Brunetta                                 */
 /*                                                                         */
 /*  This program is free software; you can redistribute it and/or modify   */
 /*  it under the terms of the GNU General Public License as published by   */
@@ -15,11 +15,10 @@
 /*  along with this program; if not, write to the Free Software            */
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */
 /*                                                                         */
-/*  Author's addresses:                                                    */
-/*  email: brunetta@tin.it, stebrune@dsi.unive.it                          */
+/*  Author's email address: stenio@brunettaeperin.it                       */
 
 static char rcsid[] = \
-   "$Id: asp.c,v 1.7 1998/02/21 16:37:24 ste Exp $";
+   "$Id: asp.c,v 1.51 2001/01/23 11:20:01 ste Exp $";
 
 #include "asp.h"
 
@@ -227,13 +226,17 @@ int match(char* buf, char* host){
 /* update_hosts trova la linea host in /etc/hosts e cambia il relativo
    indirizzo. Se addr e` NULL la linea viene eliminata. */
 void update_hosts(char* host, char* addr){
-  char buf[BUFSIZ+1], *name;
+  char buf[BUFSIZ+1], *name = HOSTS_PATH ".asp";
   FILE *fp1, *fp2;
   int c, m;
 
   if((fp1 = fopen(HOSTS_PATH, "r+")) == NULL) asp_perror("fopen");
 
-  if((name = tmpnam(NULL)) == NULL) asp_perror("tmpnam");
+  /* CPhipps 2000/02/17 - Use temporary file in /etc, so we can 
+   * rename(2) it to /etc/hosts cleanly once we're done.
+   * I'm assuming there will only ever be one copy of asp running, if 
+   * that's not true than this open should be O_EXCL and we need stale 
+   * temp file handling too. But I doubt it's needed. */
   if((fp2 = fopen(name, "w")) == NULL) asp_perror("fopen");
 
   do{ 
@@ -246,12 +249,25 @@ void update_hosts(char* host, char* addr){
     if(addr != NULL) fprintf(fp2, "%s\t%s\n", addr, host);
     while((c = fgetc(fp1)) != EOF)
       fputc(c, fp2);
-    if(remove(HOSTS_PATH) == -1) asp_perror("remove");
+    /* CPhipps 2000/02/17 - quoting from the libc docs:
+
+ One useful feature of `rename' is that the meaning of the name
+ NEWNAME changes "atomically" from any previously existing file by
+ that name to its new meaning (the file that was called OLDNAME).
+ There is no instant at which NEWNAME is nonexistent "in between"
+ the old meaning and the new meaning.  If there is a system crash
+ during the operation, it is possible for both names to still
+ exist; but NEWNAME will always be intact if it exists at all.
+
+     * /etc/hosts is an important system file.. we want it 
+     * to always exist, so we use this feature */
+                                   
     if(rename(name, HOSTS_PATH) == -1) asp_perror("rename");
   }else{
-    if(addr != NULL)
+    if(addr != NULL){
       if(strchr(buf, '\n')) fprintf(fp1, "%s\t%s\n", addr, host);
       else fprintf(fp1, "\n%s\t%s\n", addr, host);
+	 }
     if(remove(name) == -1) asp_perror("remove");
   }
 
@@ -260,7 +276,7 @@ void update_hosts(char* host, char* addr){
 }
 
 void about(){
-  printf("Copyright (C) 1996-98 Stenio Brunetta\n" \
+  printf("Copyright (C) 1996-2001 Stenio Brunetta\n" \
 	  "%s\n", rcsid);
   exit(EXIT_SUCCESS);
 }
