@@ -1,5 +1,5 @@
 /* Asp Address Search Protocol (file: asp.h)                               */
-/* Copyright (C) 1996-97 Stenio Brunetta                                   */
+/* Copyright (C) 1996-2001 Stenio Brunetta                                 */
 /*                                                                         */
 /*  This program is free software; you can redistribute it and/or modify   */
 /*  it under the terms of the GNU General Public License as published by   */
@@ -15,8 +15,7 @@
 /*  along with this program; if not, write to the Free Software            */
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */
 /*                                                                         */
-/*  Author's addresses:                                                    */
-/*  email: brunetta@tin.it, stebrune@dsi.unive.it                          */
+/*  Author's email address: stenio@brunettaeperin.it                       */
 
 
 #ifndef ASP_H
@@ -130,6 +129,9 @@ char *strchr (), *strrchr ();
 #   include <unistd.h>
 #endif
 
+#include <net/if.h>
+#include <sys/ioctl.h>
+
 #define MSGSIZE         64
 #define ASP_PORT	27374
 
