@@ -1,5 +1,5 @@
 /* Aspd Address Search Protocol Daemon                                     */
-/* Copyright (C) 1996-97 Stenio Brunetta                                   */
+/* Copyright (C) 1996-2001 Stenio Brunetta                                 */
 /*                                                                         */
 /*  This program is free software; you can redistribute it and/or modify   */
 /*  it under the terms of the GNU General Public License as published by   */
@@ -15,17 +15,17 @@
 /*  along with this program; if not, write to the Free Software            */
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */
 /*                                                                         */
-/*  Author's addresses:                                                    */
-/*  email: brunetta@tin.it, stebrune@dsi.unive.it                          */
+/*  Author's email address: stenio@brunettaeperin.it                       */
 
 static char rcsid[] = \
-           "$Id: aspd.c,v 1.7 1998/02/21 16:37:24 ste Exp $";
+           "$Id: aspd.c,v 1.39 2001/02/01 19:27:26 ste Exp $";
 
 #include "asp.h"
 
 static int rep = 5;
 static int hname_len;
 static char hname[MSGSIZE] = "";
+static char* interface = NULL;
 
 char* prog_name;
 
@@ -130,11 +130,12 @@ int asp_daemon(void){
   
   sock = make_dsocket(ntohs(psServent->s_port));
   
-  if((cpid = fork()) != 0) /* parent */
+  if((cpid = fork()) != 0){ /* parent */
     if(cpid == -1)
-	asp_perror("fork");
+		asp_perror("fork");
     else
-	exit(EXIT_SUCCESS);
+		exit(EXIT_SUCCESS);
+  }
   
   signal(SIGTERM, handler);
   signal(SIGHUP, handler);
@@ -184,7 +185,8 @@ RETSIGTYPE handler(int sig){
 
 int make_dsocket(int port){
   int sock ;
-  struct sockaddr_in name;
+  struct sockaddr_in *name;
+  struct ifreq ifr;
 
   /* Crea il socket */
   sock = socket(PF_INET, SOCK_DGRAM, 0);
@@ -192,24 +194,37 @@ int make_dsocket(int port){
   if(sock < 0) asp_perror("socket");
 
   /* Assegna il nome al socket */
-  name.sin_family = AF_INET;
-  name.sin_port = htons(port);
-  name.sin_addr.s_addr = htonl(INADDR_ANY);
+  name = (struct sockaddr_in *)&ifr.ifr_addr;
+  name->sin_family = AF_INET;
 
-  if(bind(sock, (struct sockaddr *)&name, sizeof(name)) < 0)
+
+  /* Se invocato con l'opzione -i assegna il nome con l'indirizzo
+     dell'interfaccia specificata al socket */
+  if(interface != NULL){
+    
+    bzero(&ifr, sizeof(ifr));
+    strcpy(ifr.ifr_name, interface);
+    
+    if(ioctl(sock, SIOCGIFADDR, &ifr) < 0) asp_perror("ioctl");
+
+  } else name->sin_addr.s_addr = htonl(INADDR_ANY);
+
+  name->sin_port = htons(port);
+
+  if(bind(sock, (struct sockaddr *)name, sizeof(*name)) < 0)
     asp_perror("bind");
 
   return sock;
 }
 
 void about(){
-  fprintf(stderr, "Copyright (C) 1996-98 Stenio Brunetta\n" \
+  fprintf(stderr, "Copyright (C) 1996-2001 Stenio Brunetta\n" \
 	  "%s\n", rcsid);
   exit(EXIT_SUCCESS);
 }
 
 void use(void){
-  fprintf(stderr, "Usage: %s [-r repetitions] [hostname]\n" \
+  fprintf(stderr, "Usage: %s [-r repetitions] [-i interface] [hostname]\n" \
 	  "%s -V\n", prog_name, prog_name);
   exit(EXIT_FAILURE);
 }
@@ -219,7 +234,7 @@ void parse_cmdline(int argc, char* argv[]){
 
   /* we don't want getopt to write error messages */
   opterr = 0;
-  while((c = getopt(argc, argv, "r:V")) != -1){
+  while((c = getopt(argc, argv, "i:r:V")) != -1){
     switch(c){
     case 'V':
       if(argc > 2 || strcmp(argv[argc-1], "-V"))
@@ -231,6 +246,8 @@ void parse_cmdline(int argc, char* argv[]){
         fprintf(stderr, "%s: bad repetition number\n", argv[0]);
         exit(EXIT_FAILURE);
       }
+    case 'i': /* interface */
+      interface = optarg;
       break;
     default:
       use();
